load_fasta_to_list <- function( filename ) {
	echo = function( message, ... ) {
		cat( sprintf( message, ... ))
	}
	echo( "++load_fasta_to_list: loading FASTA file from \"%s\"...\n", filename )
	if( stringr::str_ends( filename, "[.]gz" )) {
		input_file = gzfile(filename)
		data = readLines( input_file )
		close(input_file)
	} else {
		input_file = file(filename)
		data = readLines( input_file )
		close(input_file)
	}
	echo( "++load_fasta_to_list: finding contigs...\n" )
	contig_starts = grep( "^>", data )
	# Compute the contig names by stripping out '>':
	contig_names = gsub(
		'^>', '',
		data[contig_starts]
	)
	contig_names = gsub(
		' .*$', '',
		contig_names
	)
	echo( "++load_fasta_to_list: ok, %d sequences found.\n", length(contig_names))
	echo( "++load_fasta_to_list: finding sequences...\n" )
	# For each contig, find the range of sequence lines corresponding
	# to that contig
	sequence_line_ranges = purrr::map(
		1:length(contig_starts),
		function(i) {
			c(
				contig_starts[i]+1,
				c(contig_starts,length(data))[i+1]-1 )
		}
	)
	echo( "++load_fasta_to_list: ok, concatenating sequence...\n" )
	# Now use `paste()` to stick together the sequences for each contig
	result = purrr::map(
		sequence_line_ranges,
		function(range) {
			paste( data[range[1]:range[2]], collapse = "" )
		}
	)
	# Split into a character vector...
	for( i in 1:length( result )) {
		result[[i]] = strsplit( result[[i]], split = "", fixed = T )[[1]]
	}
	# Make sure to give our list the right names...
	names(result) = contig_names
	# ...and return it.
	echo( "++load_fasta_to_list: success!\n" )
	return( result )
}
